// FilterDlg.cpp :      CFilterDlg
// (C) Red User, 2004
// mailto:reduser@nm.ru

#include "stdafx.h"
#include <process.h>
#include "Filter.h"
#include "Filt.h"
#include "FiltThread.h"
#include ".\filterdlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CFilterDlg::CFilterDlg(CWnd* pParent)
	: CDialog(CFilterDlg::IDD, pParent)
    , m_strSrcFileName("")
    , m_strDestFileName("")
    , m_strCondF("")
    , m_strCondL("")
    , m_strCondB("")
    , m_bCheckFromTo(FALSE)
    , m_bCheckFromToEnd(FALSE)
    , m_bCheckMaxEmpty(FALSE)
    , m_iMaxEmpty(0)
    , m_hFiltThread(0)
{
    m_hIcon = AfxGetApp()->LoadStandardIcon(IDI_APPLICATION);
}

void CFilterDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Text(pDX, IDC_EDIT_SRC, m_strSrcFileName);
    DDX_Text(pDX, IDC_EDIT_DEST, m_strDestFileName);
    DDX_Text(pDX, IDC_EDIT_COND_F, m_strCondF);
    DDX_Text(pDX, IDC_EDIT_COND_L, m_strCondL);
    DDX_Text(pDX, IDC_EDIT_COND_B, m_strCondB);
    DDX_Check(pDX, IDC_CHECK_FROMTO, m_bCheckFromTo);
    DDX_Check(pDX, IDC_CHECK_FROMTOEND, m_bCheckFromToEnd);
    DDX_Check(pDX, IDC_CHECK_MAXEMPTY, m_bCheckMaxEmpty);
    DDX_Text(pDX, IDC_EDIT_MAXEMPTY, m_iMaxEmpty);
	DDV_MinMaxInt(pDX, m_iMaxEmpty, 0, INT_MAX / 10);
}

BEGIN_MESSAGE_MAP(CFilterDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
    ON_BN_CLICKED(IDRUN, OnBnClickedRun)
    ON_BN_CLICKED(IDC_BUTTON_BROWSESRC, OnBnClickedBrowseSrc)
    ON_BN_CLICKED(IDC_BUTTON_BROWSEDEST, OnBnClickedBrowseDest)
    ON_BN_CLICKED(IDC_CHECK_FROMTO, OnBnClickedCheckFromTo)
    ON_BN_CLICKED(IDC_CHECK_FROMTOEND, OnBnClickedCheckFromToEnd)
    ON_BN_CLICKED(IDC_CHECK_MAXEMPTY, OnBnClickedCheckMaxEmpty)
    ON_WM_TIMER()
    ON_WM_CLOSE()
END_MESSAGE_MAP()

BOOL CFilterDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CFilterDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CFilterDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CFilterDlg::OnBnClickedRun()
{
    const int iMaxSize = 0x100000;
    ULONGLONG ullSize;
    Info *pInfo = NULL;

    try
    {
        CWaitCursor wait;
        if (!UpdateData(TRUE))
        {
            m_iMaxEmpty = 0;
            UpdateData(FALSE);
            return;
        }
        ASSERT(m_iMaxEmpty >= 0);
		if (!m_strSrcFileName.GetLength()) {
			MessageBox("    .", NULL, MB_ICONWARNING);
			return;
		}
		if (!m_strDestFileName.GetLength()) {
			MessageBox("    .", NULL, MB_ICONWARNING);
			return;
		}
        CFile fSrc(m_strSrcFileName, CFile::modeRead | CFile::shareDenyWrite);        
        ullSize = fSrc.GetLength();
        if (ullSize <= iMaxSize)
        {
            //      
            pInfo = new Info;
            pInfo->pFilt = new CFilt;                        
            if (m_bCheckFromTo) 
                if (!(pInfo->pFilt->DelFromTo(m_strCondF, m_strCondL)))
                    AfxThrowMemoryException();                    
            if (m_bCheckFromToEnd)
                if (!(pInfo->pFilt->DelFromToEnd(m_strCondB)))
                    AfxThrowMemoryException();
            if (m_bCheckMaxEmpty)
                pInfo->pFilt->SetMaxEmpty(m_iMaxEmpty);  
            pInfo->pfDest = new CFile(m_strDestFileName, 
                CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite);
            pInfo->iSize = (int)ullSize;
#ifdef ONBNCLICKEDRUN_PBUF_MEMORY_ERROR
            pInfo->pBuf = new char[2000000000];
#else
            pInfo->pBuf = new char[pInfo->iSize];
#endif
            fSrc.Read(pInfo->pBuf, pInfo->iSize);
            fSrc.Close();

#ifndef ONBNCLICKEDRUN_BEGINTHREADEX_ERROR
            m_hFiltThread = (HANDLE)_beginthreadex(NULL, 0, FiltThreadFunc, pInfo, 0, NULL);
#endif
            if (!m_hFiltThread)
            {
                delete pInfo;
                MessageBox("    .", NULL, MB_ICONWARNING);
            }
            else
            {
                GetDlgItem(IDRUN)->EnableWindow(FALSE);
                GetDlgItem(IDCANCEL)->EnableWindow(FALSE);
                SetTimer(ID_FILT_TIMER, 100, NULL);
            }
        }
        else
            MessageBox("  .", NULL, MB_ICONWARNING);        
    }
    catch (CException* perr)
    {
        delete pInfo;
        perr->ReportError();
        perr->Delete();
    }
}

void CFilterDlg::OnBnClickedBrowseSrc()
{
    Browse(m_strSrcFileName, IDC_EDIT_SRC);
}

void CFilterDlg::OnBnClickedBrowseDest()
{
    Browse(m_strDestFileName, IDC_EDIT_DEST);
}

void CFilterDlg::OnBnClickedCheckFromTo()
{
    EnblDisbl(IDC_EDIT_COND_F);
    EnblDisbl(IDC_EDIT_COND_L);
    SetDlgItemText(IDC_EDIT_COND_F, "");
    SetDlgItemText(IDC_EDIT_COND_L, "");
}

void CFilterDlg::OnBnClickedCheckFromToEnd()
{
    EnblDisbl(IDC_EDIT_COND_B);
    SetDlgItemText(IDC_EDIT_COND_B, "");
}

void CFilterDlg::OnBnClickedCheckMaxEmpty()
{
    EnblDisbl(IDC_EDIT_MAXEMPTY);
    SetDlgItemText(IDC_EDIT_MAXEMPTY, "0");
}

void CFilterDlg::OnTimer(UINT nIDEvent)
{
    DWORD dwExitCode = STILL_ACTIVE;

    GetExitCodeThread(m_hFiltThread, &dwExitCode);
    if (!dwExitCode)
    {
        CloseHandle(m_hFiltThread);
        m_hFiltThread = 0;
        KillTimer(ID_FILT_TIMER);
        GetDlgItem(IDRUN)->EnableWindow(TRUE);        
        GetDlgItem(IDCANCEL)->EnableWindow(TRUE);
    }
    CDialog::OnTimer(nIDEvent);
}

void CFilterDlg::OnClose()
{
    if (m_hFiltThread)
        MessageBox("   .", NULL, MB_ICONWARNING);
    else
        CDialog::OnClose();
}

void CFilterDlg::Browse(CString &strFullFileName, int nID)
{
    CFileDialog brws(TRUE);
    brws.m_ofn.lpstrTitle = "Filter";
    brws.m_ofn.Flags |= OFN_ENABLESIZING;
    brws.m_ofn.lpstrFilter =
        " \0*.*\0"
        " HTML\0*.htm;*.html\0"
        " \0*.txt\0";
    if (brws.DoModal() == IDOK)
        strFullFileName = brws.GetPathName();
    SetDlgItemText(nID, strFullFileName);
}

void CFilterDlg::EnblDisbl(int nID)
{
    CWnd *pWnd = GetDlgItem(nID);
    ASSERT(pWnd);
    pWnd->EnableWindow(!(pWnd->IsWindowEnabled()));
}
